#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){

    vidGrabber.setVerbose(true);
    vidGrabber.initGrabber(320,240);

    colorImg.allocate(320,240);
	grayImage.allocate(320,240);
	grayBg.allocate(320,240);
	grayDiff.allocate(320,240);

	bLearnBakground = true;
	threshold = 80;
}

//--------------------------------------------------------------
void ofApp::update(){
	ofBackground(0);

    bool bNewFrame = false;
    vidGrabber.update();
    bNewFrame = vidGrabber.isFrameNew();

	if (bNewFrame){
        colorImg.setFromPixels(vidGrabber.getPixels(), 320,240);
        grayImage = colorImg;
		if (bLearnBakground == true){
			grayBg = grayImage;		// the = sign copys the pixels from grayImage into grayBg (operator overloading)
			bLearnBakground = false;
		}

		// take the abs value of the difference between background and incoming and then threshold:
		grayDiff.absDiff(grayBg, grayImage);
		grayDiff.threshold(threshold);

		// find contours which are between the size of 20 pixels and 1/3 the w*h pixels.
		// also, find holes is set to true so we will get interior contours as well....
		contourFinder.findContours(grayDiff, 20, (340*240)/3, 10, true);	// find holes
	}
}

//--------------------------------------------------------------
void ofApp::draw(){

	// draw the incoming, the grayscale, the bg and the thresholded difference
	ofSetHexColor(0xffffff);
	ofNoFill();
    ofRect(20,70,20+320,70+240);

	//colorImg.draw(20,20);
	//grayImage.draw(360,20);
	//grayBg.draw(20,280);
	//grayDiff.draw(360,280);

	// then draw the contours:
	ofFill();
	ofSetHexColor(0x333333);
	ofRect(360,540,320,240);
	ofSetHexColor(0xffffff);

	// we could draw the whole contour finder
	//contourFinder.draw(360,540);

	// or, instead we can draw each blob individually from the blobs vector,
	// this is how to get access to them:
	// finally, a report:
	ofSetHexColor(0xffffff);
	stringstream reportStr;
	reportStr << "press ' ' or +/-  " << endl
			  << "nb blobs " << contourFinder.nBlobs << endl;


   // for (int i = 0; i < contourFinder.nBlobs; i++){

        if (contourFinder.nBlobs > 0) {
        contourFinder.blobs[0].draw(20,70);

		// draw over the centroid if the blob is a hole
		ofSetColor(255);

			ofDrawBitmapString("hole",
				contourFinder.blobs[0].boundingRect.getCenter().x + 20,
				contourFinder.blobs[0].boundingRect.getCenter().y + 70);
				reportStr << "blob[0] x : " << contourFinder.blobs[0].boundingRect.getCenter().x << endl
                 << "blob[0] y : " << contourFinder.blobs[0].boundingRect.getCenter().y;


    }

	ofDrawBitmapString(reportStr.str(), 20, 20);
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){

	switch (key){
		case ' ':
			bLearnBakground = true;
			break;
		case '+':
			threshold ++;
			if (threshold > 255) threshold = 255;
			break;
		case '-':
			threshold --;
			if (threshold < 0) threshold = 0;
			break;
	}
}
